#
# Copyright (C) 2020 Arm Mbed. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
#
"""Integration with https://github.com/ARMmbed/mbed-tools."""
from mbed_build._internal.mbed_tools.configure import configure


configure = configure



#
# Copyright (C) 2020 Arm Mbed. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
#
"""Export top-level CMakeLists.txt command."""
import pathlib

import click

from typing import Any

from mbed_build._internal.cmake_file import generate_mbed_config_cmake_file
from mbed_build._internal.write_files import write_file


@click.command(
    help="Generate an Mbed OS config CMake file and write it to a .mbedbuild folder in the program directory."
)
@click.option(
    "-o",
    "--output-directory",
    type=click.Path(),
    default=".",
    help=(
        "Destination for the generated .mbedbuild/mbed_config.cmake file containing configuration parameters to build "
        "Mbed OS. The default is the current working directory."
    ),
)
@click.option(
    "-t",
    "--toolchain",
    type=click.Choice(["ARM", "GCC_ARM"]),
    required=True,
    help="The toolchain you are using to build your app.",
)
@click.option("-m", "--mbed-target", required=True, help="A build target for an Mbed-enabled device, eg. K64F")
@click.option(
    "-p",
    "--program-path",
    type=click.Path(),
    default=".",
    help="Path to local Mbed program. By default is the current working directory.",
)
def configure(output_directory: Any, toolchain: str, mbed_target: str, program_path: str) -> None:
    """Exports a mbed_config.cmake file to a .mbedbuild directory in the output path.

    The parameters set in the CMake file will be dependent on the combination of
    toolchain and Mbed target provided and these can then control which parts of
    Mbed OS are included in the build.

    This command will create the .mbedbuild directory at the output path if it doesn't
    exist.

    Args:
        output_directory: the path where .mbedbuild/mbed_config.cmake will be written
        toolchain: the toolchain you are using (eg. GCC_ARM, ARM)
        mbed_target: the target you are building for (eg. K64F)
        program_path: the path to the local Mbed program
    """
    cmake_file_contents = generate_mbed_config_cmake_file(mbed_target.upper(), program_path, toolchain)
    output_directory = pathlib.Path(output_directory, ".mbedbuild")
    write_file(output_directory, "mbed_config.cmake", cmake_file_contents)
    click.echo(f"mbed_config.cmake has been generated and written to '{str(output_directory.resolve())}'")



#
# Copyright (C) 2020 Arm Mbed. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
#
"""Configuration assembly algorithm."""
from pathlib import Path
from typing import Iterable, Optional

from mbed_build._internal.config.config import Config
from mbed_build._internal.config.cumulative_data import CumulativeData
from mbed_build._internal.config.source import Source
from mbed_build._internal.find_files import LabelFilter, filter_files, find_files
from mbed_project import MbedProgram


def assemble_config(mbed_target: str, mbed_program_directory: Path) -> Config:
    """Assemble Config for given target and program directory.

    The structure and configuration of MbedOS requires us to do multiple passes over
    configuration files, as each pass might affect which configuration files should be included
    in the final configuration.
    """
    target_source = Source.from_target(mbed_target, mbed_program_directory)
    mbed_lib_files = find_files("mbed_lib.json", mbed_program_directory)
    mbed_program = MbedProgram.from_existing(mbed_program_directory)
    mbed_app_file = mbed_program.files.app_config_file
    return _assemble_config_from_sources_and_lib_files(target_source, mbed_lib_files, mbed_app_file)


def _assemble_config_from_sources_and_lib_files(
    target_source: Source, mbed_lib_files: Iterable[Path], mbed_app_file: Optional[Path] = None
) -> Config:
    previous_cumulative_data = None
    current_cumulative_data = CumulativeData.from_sources([target_source])
    while previous_cumulative_data != current_cumulative_data:
        filtered_files = _filter_files(mbed_lib_files, current_cumulative_data)
        mbed_lib_sources = [Source.from_mbed_lib(file, current_cumulative_data.labels) for file in filtered_files]
        all_sources = [target_source] + mbed_lib_sources
        if mbed_app_file:
            all_sources = all_sources + [Source.from_mbed_app(mbed_app_file, current_cumulative_data.labels)]
        previous_cumulative_data = current_cumulative_data
        current_cumulative_data = CumulativeData.from_sources(all_sources)

    return Config.from_sources(all_sources)


def _filter_files(files: Iterable[Path], cumulative_data: CumulativeData) -> Iterable[Path]:
    filters = (
        LabelFilter("TARGET", cumulative_data.labels),
        LabelFilter("FEATURE", cumulative_data.features),
        LabelFilter("COMPONENT", cumulative_data.components),
    )
    return filter_files(files, filters)



#
# Copyright (C) 2020 Arm Mbed. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
#
"""Entrypoint for development purposes."""
import click

from mbed_build.mbed_tools import configure


@click.group()
def cli() -> None:
    """Group exposing the commands from the mbed-build package."""
    pass


cli.add_command(configure)

cli()
